# Generated by roxygen2: do not edit by hand

export(autocorr.mat)
export(averageCorr)
export(averageCorrSq)
export(cov_transform)
export(decorrelate)
export(dmult)
export(eclairs)
export(eclairs_corMat)
export(eclairs_sq)
export(effVariance)
export(getCor)
export(getCov)
export(getShrinkageParams)
export(getWhiteningMatrix)
export(lm_each_eclairs)
export(lm_eclairs)
export(logDet)
export(mahalanobisDistance)
export(mult_eclairs)
export(quadForm)
export(reform_decomp)
export(rmvnorm_eclairs)
export(sumInverseCorr)
export(sv_threshold)
export(tr)
export(whiten)
exportClasses(eclairs)
exportClasses(fastcca)
exportMethods(getCor)
exportMethods(getCov)
exportMethods(kappa)
exportMethods(plot)
import(Rcpp)
import(stats)
importFrom(CholWishart,lmvgamma)
importFrom(Matrix,crossprod)
importFrom(Matrix,diag)
importFrom(Matrix,isSymmetric)
importFrom(Matrix,t)
importFrom(Matrix,tcrossprod)
importFrom(Rfast,colVars)
importFrom(Rfast,eachrow)
importFrom(Rfast,matrnorm)
importFrom(Rfast,standardise)
importFrom(graphics,arrows)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(irlba,irlba)
importFrom(irlba,partial_eigen)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,show)
importFrom(stats,cor)
importFrom(stats,cov2cor)
importFrom(stats,median)
importFrom(stats,rchisq)
importFrom(utils,head)
useDynLib(decorrelate)
